/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  EthernetAddress (implementation)
    Package: tcpip
    File:    EthernetAddress.java


*/
package fi.hut.tcm.jacob.tcpip;
import java.util.StringTokenizer;

/**
    EthernetAddress
    
    A utility class for storing and converting ethernet addresses.
    XXX Exceptions missing - eg. when an illegal string is given to
    the constructor..
*/
public class EthernetAddress {
    private long mac;

    public EthernetAddress(long addr) {
    //+ EthernetAddress::EthernetAddress(long)
      mac = addr;
    //- EthernetAddress::EthernetAddress(long)
    }

    public EthernetAddress(String strAddr) {
    //+ EthernetAddress::EthernetAddress(String)
      StringTokenizer addrTok = new StringTokenizer(strAddr, ":", false);
      if (addrTok.countTokens() != 6) {
        throw new
	  IllegalArgumentException("Invalid ethernet address in argument: " + strAddr);
      }
      mac = 0;
      long multiply = 1099511627776L;  // 256 ^ 5, REMEMBER THE 'L' !
      for (int i = 0; i < 6; i++) {
        try {
          mac += multiply * (long) (Integer.parseInt(addrTok.nextToken(), 16 ) & 0xff);
	  multiply /= 256;
        } catch (NumberFormatException e) {
        throw new
	  IllegalArgumentException("Invalid ethernet address in argument: " + strAddr);
        }
      }
    //- EthernetAddress::EthernetAddress(String)
    }

    public static byte[] toArray(long addr) {
    //+ EthernetAddress::toArray(long)
      byte[] macarr = new byte[6];
      for (int i = 5; i >= 0; i--) {
	macarr[i] = (byte) ((addr % 256) & 0xff);
	addr /= 256;
      }
      return macarr;
    //- EthernetAddress::toArray(long)
    }

    public byte[] toArray() {
    //+ EthernetAddress::toArray( )
      return toArray(mac);
    //- EthernetAddress::toArray( )
    }

    public static String toString(long addr) {
    //+ EthernetAddress::toString(long)
      String elem;
      byte[] macarr = EthernetAddress.toArray(addr);
      String macstr = new String(Integer.toHexString( macarr[0] & 0xff ));
      if (macstr.length() < 2)
	macstr = "0" + macstr;
      for (int i = 1; i < 6; i++) {
	elem = Integer.toHexString( macarr[i] & 0xff );
	if (elem.length() < 2)
	  elem = "0" + elem;
	macstr = macstr + ":" + elem;
      }
      return macstr;
    //- EthernetAddress::toString(long)
    }

    public String toString() {
    //+ EthernetAddress::toString( )
      return toString(mac);
    //- EthernetAddress::toString( )
    }

    public long toLong() {
    //+ EthernetAddress::toLong( )
      return mac;
    //- EthernetAddress::toLong( )
    }

}; //EthernetAddress

