/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpStubMessenger (implementation)
    Package: ip
    File:    IpStubMessenger.java


*/
package fi.hut.tcm.jacob.tcpip;
import fi.hut.tcm.jacob.conduits.*;



/**
  IpStubMessenger
    
  */
public class IpStubMessenger
  implements TcpipStubMessenger {

  public static final IpStubMessenger instance = new IpStubMessenger();
    
  private IpStubMessenger() {
  }

  // Should be never called
  public void apply(Message message, State state, Session session) {}

  public void printHeader(Message message) {
    //+ IpStubMessenger::printHeader(Message)

    BinaryBuffer buffer = message.getBuffer();
    
    // Create header dump with field names
    int vers = (buffer.getByte(0) >> 4) & 0x0f;
    int hlen = buffer.getByte(0) & 0x0f;
    int servtype = buffer.getByte(1);
    int tlen = buffer.getShort(2);
    int id = buffer.getShort(4);
    int flags = (buffer.getByte(6) >> 5) & 0x07;
    int fragoffs = buffer.getShort(6) & 0x1fff;
    int ttl = buffer.getByte(8);
    int protid = buffer.getByte(9);
    int csum = buffer.getShort(10);
    int srcIpAddr = buffer.getInt(12);
    int destIpAddr = buffer.getInt(16);
    
    // Print
    System.err.println("IPV: " + vers + "  HLEN: " + hlen +
		       "  STYPE: " + servtype + "  TLEN: " + tlen);
    System.err.println("ID: " + id + "  FLAGS: " + flags + 
		       "  FRAGOFFS: " + fragoffs);
    System.err.println("TTL: " + ttl + "  PROT.ID: " + protid +
		       "  CSUM: " + csum);
    System.err.print("SRC IP: " + 
		     ((srcIpAddr >> 24) & 0xff) + "." + 
		     ((srcIpAddr >> 16) & 0xff) + "." + 
		     ((srcIpAddr >>  8) & 0xff) + "." + 
		     ((srcIpAddr >>  0) & 0xff) + "   "); 
    System.err.println("DST IP: " + 
		       ((destIpAddr >> 24) & 0xff) + "." +
		       ((destIpAddr >> 16) & 0xff) + "." +
		       ((destIpAddr >>  8) & 0xff) + "." +
		       ((destIpAddr >>  0) & 0xff));
    //- IpStubMessenger::printHeader(Message)
    }    
}; //IpStubMessenger

