/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  UdpAdaptor (implementation)
    Package: ip
    File:    UdpAdaptor.java


*/
package fi.hut.tcm.jacob.tcpip;
import fi.hut.tcm.jacob.tcpip.udp.*;
import fi.hut.tcm.jacob.conduits.*;
import java.net.*;
import fi.hut.tcm.jacob.tcpip.JacobDatagramSocket;
/**
    UdpAdaptor
    
    
*/
public final class UdpAdaptor
  extends Adaptor {

    private static int MAX_DATAGRAM_SIZE = 65535;
    private static JacobDatagramSocket socket;

    public UdpAdaptor(JacobDatagramSocket socket) {
    //+ UdpAdaptor::UdpAdaptor(JacobDatagramSocket)
      this.socket = socket;
    //- UdpAdaptor::UdpAdaptor(JacobDatagramSocket)
    }

    public void adapt(Message message) {
    //+ UdpAdaptor::adapt(Message)
      BinaryBuffer buffer = message.getBuffer();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      int len = buffer.getLength();
      byte[] data = new byte[len];
      buffer.exportData(data, 0, len);

      int srcIpAddr = oob.getSrcIpAddr();
      String srcIpStr = new String(((srcIpAddr >> 24) & 0xff) + "." + 
				   ((srcIpAddr >> 16) & 0xff) + "." + 
				   ((srcIpAddr >>  8) & 0xff) + "." + 
				   ((srcIpAddr >>  0) & 0xff)); 
      try {
	DatagramPacket packet = 
	  new DatagramPacket(data, 
			     data.length, 
			     InetAddress.getByName(srcIpStr),
			     oob.getSrcPort());
	socket.put(packet);
      } catch (UnknownHostException e) {
	System.err.println("Host " + srcIpStr + 
			   " caused an UnknownHostException");
      } 
      //- UdpAdaptor::adapt(Message)
    }
    
    public void send(DatagramPacket packet) {
    //+ UdpAdaptor::send(DatagramPacket)
      byte[] data = packet.getData();

      BinaryBuffer buffer = 
	BinaryBuffer.newBinaryBuffer(MAX_DATAGRAM_SIZE, 200, data.length);
      TcpipOutOfBand oob = new TcpipOutOfBand();
      
	/*
	 * The first getAddress() returns an InetAddress object,
	 * the second one a byte array.
	 */
	byte[] ipAddr = packet.getAddress().getAddress();
	int destIpAddr = 
	  ((int) ipAddr[0] & 0xff) << 24 |
	  ((int) ipAddr[1] & 0xff) << 16 |
	  ((int) ipAddr[2] & 0xff) <<  8 |
	  ((int) ipAddr[3] & 0xff) <<  0;
	int destPort = packet.getPort();
 
	// Copy buffer data
	buffer.importData(data, 0, data.length);

	// Copy out-of-band data
	oob.setDestIpAddr(destIpAddr);
	oob.setDestPort(destPort);

	Message msg = 
	  new Message(UdpHeaderToNetMessenger.instance, oob, buffer);

        if (sideA != null) {
          MessageTransporter trans = 
            MessageTransporter.newMessageTransporter(sideA, msg, this);
          SimpleScheduler.instance.makeRunnable(trans);
        }

      }
      //- UdpAdaptor::send(DatagramPacket)
}; //UdpAdaptor


