/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  Arp (implementation)
    Package: arp
    File:    Arp.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Hashtable;

/**
    Arp
    
    ARP (Address Resolution Protocol).<br>
    - Uses Ethernet broadcast to find Ethernet hardware address of any
    locally connected host with given IP address.<br>
    - Forwards and receives packets to and from the ARP/IP STUB and to the
    EthernetHeaderSession. <br>
    - Provides an interface for the user to the ARP Table. <br>
    - Basically, just encapsulates the ARP-related Conduits. <br>
    
    See also: ArpTable.
*/
public class Arp
  extends Protocol {
    /**
        Size of ARP-table
    */
    public static final int ArpTableSize = 4;
    public static final int RequestTableSize =32;
    /**
        Size of IP-address
    */
    public static final int SIZEOFIPADDR = 4;
    /**
        Size of EtherNet address
    */
    public static final int SIZEOFETHERADDR = 6;
    /**
        Size of Ethernet header without preamble.
    */
    public static final int SIZEOFETHERHEADER = 14;
    /**
        Arp Table return code: Ethernet address found
    */
    public static final byte ARPOK = 0;
    /**
        Arp Table return code: Ethernet address not found
    */
    public static final byte ARPNOTINTABLE = -1;
    /**
        Arp Table return code: failure
    */
    public static final byte ARPFAILURE = -2;
    /**
        ARP Table return code: Ethernet address found but old
    */
    public static final byte ARPOKBUTOLD = 1;
    /**
        Time (milli seconds) how long Arp Table entry 
        (IP address) is concidered unreachable until
        new request can be done.
    */
    public static final long ARPFAILEDTIME = 2*60*1000;
    /**
        Time (milli seconds) after which Arp Table entry
        expires.
    */
    public static final long ARPEXPIREDTIME = 30*60*1000;
    /**
        Time (milli seconds) after which Arp Table entry
        is still valid but should be checked soon.
    */
    public static final long ARPOKBUTOLDTIME = 20*60*1000;
    /**
        Arp Table code: entry is immutable.
    */
    public static final byte ARPIMMUTABLE = 1;
    /**
        Arp Table code: entry is mutable.
    */
    public static final byte ARPMUTABLE = 2;
    /**
        The different parts of the ARP protocol consult
        this variable for their debug level.The levels
        are as follows:<br>
        <ul>
          <li> 0 - no debug output
          <li> 1 - informational messages
          <li> 2 - detailed debug printout, not for end-users
          <li> 3 - print everything
        </ul>
    */
    public static int DebugLevel = 0;

    /**
        ARP-table can't be cloned.
    */
    protected void deepcopy()
    throws CloneNotSupportedException {
    //+ Arp::deepcopy( )
      throw new CloneNotSupportedException("XXX");
    //- Arp::deepcopy( )
    }

}; //Arp

