/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  ArpRequestDataMessenger (implementation)
    Package: arp
    File:    ArpRequestDataMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Hashtable;


/**
    ArpRequestDataMessenger
    
    Methods for making ARP request (and storing message untill
    reply or timeout is  received, not implemented)
*/
public class ArpRequestDataMessenger
  implements Messenger {
    public static ArpRequestDataMessenger instance = new ArpRequestDataMessenger();

    /**
        Not implemented, planned functionality: inserts message into
        requestTable until reply is
        received for ARP request (or Timeout)
    */
    public void enqueueMessage(Message message, Hashtable reqTable) {
    //+ ArpRequestDataMessenger::enqueueMessage(Message, Hashtable)

      int ipAddr;
      
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      ipAddr = oob.getDestIpAddr();
      // message.stopTransportation();  // XXX - cms: TODO
      
      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("ARPRequestDataMessenger.enqueueMessage: throwing away packet!");
      }

      //reqTable.put(ipAddr,message);  XXX - this is just food for thought
  
    //- ArpRequestDataMessenger::enqueueMessage(Message, Hashtable)
    }

    /**
        Generates ARP request message for ipAddr.
        
        @param message
        @param ifTable
        @param ipAddr
        @param my_session
    */
    public void makeArpRequest(Message message, TcpipInterfaceTable ifTable, int ipAddr, ArpRequestSession my_session) {
    //+ ArpRequestDataMessenger::makeArpRequest(Message, TcpipInterfaceTable, int, ArpRequestSession)

      int SIZEOFARPMESSAGE = 28;

      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      TcpipOutOfBand newoob = new TcpipOutOfBand();

      // Been having problems with long constants,
      // that's why the three steps:
      long broadcastAddr = 0xFFFFFF;
      broadcastAddr = broadcastAddr << 24;
      broadcastAddr += 0xFFFFFF;

      //create binary buffer and oob before actual message     
      Object newbuf =
	BufferMessenger.newBuffer(128, 64, SIZEOFARPMESSAGE); // 64 = plenty of room
      int len = UtilMsgr.i.getLength(newbuf);

      long srcEtherAddr;
      int srcIpAddr;
      int dstIpAddr;

      //dstIpAddr = oob.getDestIpAddr();  // This is unknown
      srcIpAddr = oob.getSrcIpAddr();

      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("ARPRequestDataMessenger: srcIpAddr = " +
			 IpAddress.toString(srcIpAddr));
      }
	
      
      TcpipInterfaceEntry ifEntry = ifTable.get( srcIpAddr );
      if ( ifEntry != null ) {
	srcEtherAddr = ifEntry.getEthAddr();
	
	newoob.setSrcEthAddr(srcEtherAddr);
	
	// XXX Inefficient - if we are just refreshing an ARP entry, then we should
	// not use ethernet broadcast, instead we should go and ask the host directly.
	// Only if there is no response should we revert to a broadcast.
	newoob.setDestEthAddr(broadcastAddr); //Always broadcast.
	
	newoob.setEthFrameType( 0x0806 );

	// 0x0001 = hardware Ethernet, 0x0800 = protocol IP:
	UtilMsgr.i.setInt(newbuf, 0, 0x00010800);

	// length of hardware addr (ethernet = 6), length of protocol address (IP = 4)
	// operation code (0x0001 = ARP Request)
	UtilMsgr.i.setInt(newbuf, 1, 0x06040001);

	UtilMsgr.i.set6BytesUnaligned(newbuf, 8, srcEtherAddr); //src Ethernet address
	UtilMsgr.i.setIntUnaligned(newbuf, 14, srcIpAddr);      //src IP address
	UtilMsgr.i.setIntUnaligned(newbuf, 24, ipAddr);   // Target IP address for which we search binding

	Message newmsg = 
	  Message.newMessage(newbuf, EthernetHeaderToNetMessenger.instance,
					    newoob);
	my_session.injectMessage("SideA", newmsg);

	if (Arp.DebugLevel > 0) {  // #Debug#
	  System.out.println("Sent an ARP request for the IP address " +
			     IpAddress.toString(ipAddr));
	}
      } else {
	System.err.println("Internal error in ARPRequestDataMessenger.makeARPRequest:");
	System.err.println("Trying to send a packet with a source IP address we do not own.");
      }

    //- ArpRequestDataMessenger::makeArpRequest(Message, TcpipInterfaceTable, int, ArpRequestSession)
    }

    public void apply(Message message, State state, Session session) {
    //+ ArpRequestDataMessenger::apply(Message, State, Session)
      ArpRequestState arpRequestState = (ArpRequestState) state;

      ArpRequestSession arpRequestSession = (ArpRequestSession) session;

      arpRequestState.applyData(this, message, arpRequestSession.requestTable,
				arpRequestSession.ifTable, arpRequestSession); 
    //- ArpRequestDataMessenger::apply(Message, State, Session)
    }

}; //ArpRequestDataMessenger

