/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  ArpRequestSession (implementation)
    Package: arp
    File:    ArpRequestSession.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import java.util.Hashtable;
import fi.hut.tcm.jacob.tcpip.*;

/**
    ArpRequestSession
    
    - Sends ARP request for IP-addresses not found in the ARP Table.
    Inserts found addresses in the ARP table. <br>
    - This session receives packets without next-hop Ethernet addresses,
    retrieves the addresses, updates the
         ARPTable and sends the packets back to the EtherAddrMUX. <br> 
     - The ARPRequestDataMessenger messenger provides methods for queueing
    packets and issuing ARP requests:
         enqueueMessage() and makeARPRequest. The entry point in the
    ARPRequestState for this messenger is
         applyData(). <br>
     - The ARPRequestResponseMessenger provides methods for removing
    packets from the queue and returning
         them to the EtherAddrMux, UpdateARPTable() and dequeueMessage().
    The entry point in the
         ARPRequestState for this messenger is applyResponse(). <br>
    - Before generating an ARP failed entry in the ARPTable, the ARP
    request must be re-attempted at least once. <br>
    - The timeouts use the Conduit timeout mechanism. 
    
*/
public class ArpRequestSession
  extends Session {
    /**
        Messages waiting for Arp reply are stored here.
    */
    public Hashtable requestTable;
    public TcpipInterfaceTable ifTable;
    public ArpTable arpTable;

    /**
        Constructor for ArpRequestSession.
    */
    public ArpRequestSession(ArpTable arptable, State initialState, TcpipInterfaceTable ifaceTbl) {
    //+ ArpRequestSession::ArpRequestSession(ArpTable, State, TcpipInterfaceTable)
      super(initialState);
      this.requestTable = new Hashtable(Arp.RequestTableSize);
      ifTable = ifaceTbl;
      arpTable = arptable;
    //- ArpRequestSession::ArpRequestSession(ArpTable, State, TcpipInterfaceTable)
    }

    /**
        Direction = "SideA" or "SideB"
    */
    public void injectMessage(String direction, Message mesg) {
    //+ ArpRequestSession::injectMessage(String, Message)

      if (direction.equals("SideA") ) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.A], mesg);
	SimpleScheduler.instance.makeRunnable(mt);

	if (Arp.DebugLevel > 2) {  // #Debug#
	  System.out.println("+++ ArpRequestSession: injected to side A");
	}
      } else if (direction.equals("SideB")) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.B], mesg);
	SimpleScheduler.instance.makeRunnable(mt);

	if (Arp.DebugLevel > 2) {  // #Debug#
	  System.out.println("+++ ArpRequestSession: injected to side B");
	}
      } else {
	System.err.println("ArpRequestSession.injectMessage: Error:" +
			   " invalid direction: " + direction);
      }
     
    //- ArpRequestSession::injectMessage(String, Message)
    }

}; //ArpRequestSession

