/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  EthernetAdaptor (implementation)
    Package: ethernet
    File:    EthernetAdaptor.java


*/
package fi.hut.tcm.jacob.tcpip.ethernet;
import fi.hut.tcm.jacob.conduits.Adaptor;
//import fi.hut.tcm.jacob.conduits.MediaAdaptor;
import fi.hut.tcm.jacob.conduits.SimpleScheduler;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.arp.*;
import java.util.*;

/**
    EthernetAdaptor
    
    
*/
public class EthernetAdaptor
  extends Adaptor
  implements Runnable {
    public static final byte[] ETHERTYPE_IP = new byte[] { (byte)0x08, (byte)0x00 };
    public static final byte[] ETHERTYPE_ARP = new byte[] { (byte)0x08, (byte)0x06 };
    public static final byte[] ETHERTYPE_RARP = new byte[] { (byte)0x80, (byte)0x35 };
    public static final byte[] ETHERTYPE_IPV6 = new byte[] { (byte)0x86, (byte)0xdd };
    public static final int ETHER_MTU = 1500;
    public static final int ETHER_ADDR_LEN = 6;
    /**
        Source and destination addresses, ethernet type.
    */
    public static int ETHER_HEADER_LEN = 14;
    private Thread adaptorThread;
    private int handle = -1;
    private String device;
    private int ppa;
    private int sap;
    private Vector filterIpAddrs;
    /**
        Temporary storage for incoming packets.
    */
    private byte[] inputBuffer = new byte[ETHER_MTU + ETHER_HEADER_LEN];
    public byte[] ownEtherAddr;

    /**
        Creates new EthernetAdaptor and binds it to receive messages of the
        given protocol.
        @param hwAddr The ethernet address of the adaptor
        @param filterIpAddrs A vector of IP addresses to filter out from
        the incoming traffic. The address(es) of the machine we are running
        on should be included in this list.
        @param device Name of the underlying device driver, e.g. "/dev/le"
        @param ppa Physical point of attachement, usually 0
        
    */
    public EthernetAdaptor(EthernetAddress hwAddr, Vector filterIpAddrs, String device, int ppa) {
    //+ EthernetAdaptor::EthernetAdaptor(EthernetAddress, Vector, String, int)
      this.device = device;
      this.ppa    = ppa;
      // The native function address() could perhaps have been used here instead: 
      ownEtherAddr = hwAddr.toArray();
      this.filterIpAddrs = filterIpAddrs;

      // Promiscuous mode?
      //bind(ethertype, false);
      bind(ETHERTYPE_IP, true);  // first parameter is a dummy variable here
    //- EthernetAdaptor::EthernetAdaptor(EthernetAddress, Vector, String, int)
    }

    /**
        Binds this EthernetAdaptor to the actual underlying Ethernet
        driver. The Adaptor will start to deliver messages of the given
        ethertype, or, if <code>promisc == true</code>, all messages.
    */
    public void bind(byte[] ethertype, boolean promisc) {
    //+ EthernetAdaptor::bind(byte[], boolean)
      if (ethertype.length != 2) 
	throw new IllegalArgumentException("ethertype.length != 2");
      if (handle != -1) 
	throw new IllegalStateException("Attempt to bind an bound " +
					"EthernetAdaptor");
      sap = (((int)ethertype[0]) & 0xff) << 8 | (((int)ethertype[1]) & 0xff);
      handle = open(device.getBytes(), ppa, sap, promisc);
      if (handle <= 0)
	throw new InternalError("XXX");
      adaptorThread = new Thread(this);

      // New: lower priority:
      int priority = Thread.MIN_PRIORITY;

      adaptorThread.setPriority(priority);
      System.out.println("priority set to " + adaptorThread.getPriority());
      adaptorThread.start();
    //- EthernetAdaptor::bind(byte[], boolean)
    }

    public byte[] getHardwareAddress() {
    //+ EthernetAdaptor::getHardwareAddress( )
      byte[] addr = new byte[ETHER_ADDR_LEN];
      if (address(addr) < 0) 
	throw new InternalError("XXX");
      return addr;
    //- EthernetAdaptor::getHardwareAddress( )
    }

    private native int address(byte[] addr);

    /**
        Stops the internal thread and releases the physical ethernet
        connection.
    */
    public void finalize() {
    //+ EthernetAdaptor::finalize( )
      // Kill the internal thread
      try {
	adaptorThread.stop();
	adaptorThread.join(100); // Wait until it is dead
      } catch (InterruptedException ex) {
      }
      adaptorThread = null;

      // Close physical interface
      if (handle != -1)
	close(handle);
      handle = -1;
    //- EthernetAdaptor::finalize( )
    }

    private native int open(byte[] device, int ppa, int sap, boolean promisc);

    private native void close(int handle);

    private int send(byte[] packet) {
    //+ EthernetAdaptor::send(byte[])
      // XXX XXX XXXXXX WORKAROUND:
      return send(packet, 0, packet.length - ETHER_HEADER_LEN);
      //return send(packet, 0, packet.length);
    //- EthernetAdaptor::send(byte[])
    }

    private native int send(byte[] packet, int offset, int length);

    private int receive(byte[] packet) {
    //+ EthernetAdaptor::receive(byte[])
      // workaround: 
      return receive(packet, 0, packet.length - ETHER_HEADER_LEN);
    //- EthernetAdaptor::receive(byte[])
    }

    private native int receive(byte[] packet, int offset, int maxlength);

    public void adapt(Message message) {
    //+ EthernetAdaptor::adapt(Message)

      Object buf = message.getContents();
      byte[] packet = UtilMsgr.i.extractData(buf);

      /*
      long ethDst = UtilMsgr.i.get6BytesUnaligned(buf, 0);
      long ethSrc = UtilMsgr.i.get6BytesUnaligned(buf, 6);
      */

      send(packet);

    //- EthernetAdaptor::adapt(Message)
    }

    /**
        Called on adaptorThread. Tries to receive packets from the physical
        interface and insert them into the runqueue.
    */
    public void run() {
    //+ EthernetAdaptor::run( )
      System.out.println("Ethernet thread started");
    forever_loop:
      for (;;) {
	
	// Some changes have been made here. First and foremost, we
	// try to throw away traffic that doesn't interest us in
	// an economic fashion.

        int len = receive(inputBuffer);
	len = len + ETHER_HEADER_LEN;  // Bug-work-around

	int etherType = (( inputBuffer[12]  & 0xff) << 8) |
	                (( (int) inputBuffer[13] ) & 0xff);

	// We are only interested in IPv4 and ARP packets:
	if ( (etherType != 0x0800) && (etherType != 0x0806) ) {
	  continue forever_loop;
	}

	/* // Debug output:
	System.out.println("Got ethernet packet with ETHERTYPE = " + etherType);
	*/
	
	boolean ours = true;
	boolean bcast = true;

      broadcast_search:
	for (int i = 0; i < 6; i++) {
	  if ( (inputBuffer[i] & 0xff) != 0xff) {
	    bcast = false;
	    break broadcast_search;
	  }
	}

      ours_search:
	for (int i = 0; i < 6; i++) {
	  if ( inputBuffer[i] != ownEtherAddr[i]) {
	    ours = false;
	    break ours_search;
	  }
	}

	if ( ! (ours || bcast) )  // intended for us on ether level?
	  continue forever_loop;

	
	if (etherType == 0x0800) {  // We got an IP packet
	  if (len >= 34) { // Ethernet = 14, IP header >= 20 octets

	    byte[] ethAddr;
	    int jj;
	    Enumeration addrs = filterIpAddrs.elements();
	    while (addrs.hasMoreElements()) {
	      ethAddr = ((IpAddress) addrs.nextElement()).toArray();
	      jj = 0;
	      while ((jj < 4) && (inputBuffer[jj + 30] == ethAddr[jj]) )
		jj++;
	      if (jj == 4)
		continue forever_loop;  // a filtered address match!
	    }

	  }
	}

	// From here on, the packet should be interesting to us, also
	// it shouldn't be about a login session we have (nice feedback loop!)
	
        Object buf = 
          BufferMessenger.newBuffer(ETHER_MTU + ETHER_HEADER_LEN,
				    0, inputBuffer, len);

        long ethDst = UtilMsgr.i.get6BytesUnaligned(buf, 0);
        long ethSrc = UtilMsgr.i.get6BytesUnaligned(buf, 6);
	/* Debug output:
        System.out.println("EthernetAdaptor: incoming packet: dst: " +
			   EthernetAddress.toString(ethDst) +
                           " src: " + EthernetAddress.toString(ethSrc) +
			   " length: " + len);
	*/

	TcpipOutOfBand oob = new TcpipOutOfBand();
        Message message = 
          Message.newMessage( buf, EthernetHeaderFromNetMessenger.instance, oob );
        if (sides[Side.A] != null) {
          MessageTransporter mt = 
            MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(),
						     message);
          SimpleScheduler.instance.makeRunnable(mt);
        }
	
	// Now we have inserted a new packet, let it be transported right away:
	Thread.currentThread().yield();
      }
    //- EthernetAdaptor::run( )
    }

    static {
    //+ EthernetAdaptor::static( )
      System.loadLibrary("EthernetAdaptor");
    //- EthernetAdaptor::static( )
    }

}; //EthernetAdaptor

