/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  Icmp (implementation)
    Package: icmp
    File:    Icmp.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;

/**
    Icmp
    
    <p>The Internet Control Message Protocol. This protocol is a required
    part of any IP implementation, even though it logically is a higher
    level protocol, using IP as its transport.<br>
    This conduit encapsulates the other ICMP modules and effectively
    acts as an interface for other protocols wanting to use the ICMP
    protocol.
*/
public class Icmp
  extends Protocol {
    /**
        Specifies the byte offset of the <code>ICMP</code> type in the
        <code>ICMP</code> message.
    */
    public static int TYPE_OFFSET = 0;
    /**
        Specifies the byte offset of the <code>ICMP</code> code in the
        <code>ICMP</code> message.
    */
    public static int CODE_OFFSET = 1;
    /**
        Specifies the byte offset of the <code>ICMP</code> checksum in the
        <code>ICMP</code> message.
    */
    public static int CHECKSUM_OFFSET = 2;
    /**
        The debug output level (0, 1, 2 or 3). Level 0 is no debug output,
        whereas level 3 prints the most debug output. Level 1 is intended
        to be informational,
        levels 2 and 3 give detailed debug information.
    */
    public static int DebugLevel = 1;

    /**
        Creates an ICMP protocol and connects its <code>sideA</code> and
        <code>sideB</code> to the given conduits.
        
        @param insideA    the first conduit to attach
        @param insideB    the second conduit to attach
    */
    public Icmp(Conduit insideA, Conduit insideB) {
    //+ Icmp::Icmp(Conduit, Conduit)
      super(insideA, insideB);
    //- Icmp::Icmp(Conduit, Conduit)
    }

    /**
        Creates a deep copy of this protocol by replacing all instance
        variables with references to <bold>copies</bold> of their contents.
    */
    public void deepcopy() {
    //+ Icmp::deepcopy( )
    //- Icmp::deepcopy( )
    }

}; //Icmp

