/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpSelectorAccessor (implementation)
    Package: icmp
    File:    IcmpSelectorAccessor.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpSelectorAccessor
    
    <p>This accessor examines the ICMP type of the message. The key is the
    same as the ICMP type for all ICMP types except ICMP Echo and ICMP Echo
    Reply. The reason for this is that incoming and outgoing Echo Requests
    (and replies) have different routes.
*/
public class IcmpSelectorAccessor
  implements Accessor {

    /**
        Gets the key of a <code>Message</code>, in order to be able to
        route it correctly. In this version though, it does not yet do it,
        only outputs debug information.
        
        @param    message  which <code>Message</code>
        @return    returns the corresponding key
    */
    public Object getKey(Message message) {
    //+ IcmpSelectorAccessor::getKey(Message)
      BinaryBuffer buf = message.getBuffer();
      int len = buf.getLength(); 

      int type = buf.getByte(0);
      
      // The b[] side of the mux are _different kinds_ of sessions,
      // so we have to change the messengers accordingly.
      // This is not very clean: this accessor has to know about
      // all the different types.
      // Perhaps they should all be adaptors, that don't care about
      // their messenger at all?

      // right now we do nothing.
      if (Icmp.DebugLevel > 2) {  // #Debug#
	System.out.println("In IcmpSelectorAccessor.getKey, key is " + type);
      }
      return new Integer(type);
    //- IcmpSelectorAccessor::getKey(Message)
    }

    /**
        Sets the key for a message, but right now, does nothing but print
        debug output.
        
        @param    message  which <code>Message</code>
        @param    key  the key for the <code>Message</code>
    */
    public void setKey(Message message, Object key) {
    //+ IcmpSelectorAccessor::setKey(Message, Object)
      BinaryBuffer buf = message.getBuffer();
      int len = buf.getLength(); 

      int type = ((Integer) key).intValue();
      // Ugly hack: here we reverse the types for Echo and Echo Reply:
      if (type == 0) {
	type = 8;
      } else if (type == 8) {
	type = 0;
      }
      buf.setByte(0, type);
    //- IcmpSelectorAccessor::setKey(Message, Object)
    }

}; //IcmpSelectorAccessor

