/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpHeaderState (implementation)
    Package: ip
    File:    IpHeaderState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpHeaderState
    
    These are the states of IpHeaderSession. For outgoing messages, the
    Messenger's addIpHeader() method is called. For incoming messages, the
    Messenger's removeIpHeader() method is called.
    
*/
public class IpHeaderState
  implements State {

  public static final IpHeaderState instance = new IpHeaderState();


    public void applyToNet(Message message, Session session, Messenger messenger) {
    //+ IpHeaderState::applyToNet(Message, Session, Messenger)
      IpHeaderToNetMessenger msgr = (IpHeaderToNetMessenger) messenger;
      IpHeaderSession sess = (IpHeaderSession) session;
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      msgr.addIpHeader(message);

      int hlen = 5; // XXX No options yet, so the header length is fixed
      int tlen = message.getBuffer().getLength();
      msgr.setVers(message, IP.IP_VERSION);
      msgr.setHLen(message, hlen);
      // XXX No options yet, so the service type is fixed
      msgr.setServiceType(message, 0);
      // XXX No options yet, so the total length is fixed
      msgr.setTotalLength(message, tlen);
      msgr.setIdentification(message, sess.getNextDatagramId());
      // XXX No options yet, so the time-to-live is fixed
      msgr.setTTL(message, 50);
      msgr.setDestIpAddr(message, oob.getDestIpAddr());

      message.setMessenger(IpReassemblyToNetMessenger.instance);
    //- IpHeaderState::applyToNet(Message, Session, Messenger)
    }

    public void applyFromNet(Message message, Messenger messenger) {
    //+ IpHeaderState::applyFromNet(Message, Messenger)
      IpHeaderFromNetMessenger msgr = (IpHeaderFromNetMessenger) messenger;
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      oob.setSrcIpAddr(msgr.getSrcIpAddr(message));
      oob.setDestIpAddr(msgr.getDestIpAddr(message));
      msgr.removeIpHeader(message);
    //- IpHeaderState::applyFromNet(Message, Messenger)
    }

}; //IpHeaderState
