/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpReassemblyState (implementation)
    Package: ip
    File:    IpReassemblyState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;

/**
    IpReassemblyState
    
    A state responsible of reassembling fragmented incoming IP packets.
    Ignores outgoing packets, except for updating their messenger
    reference..
*/
public class IpReassemblyState
  implements State {

  public static final IpReassemblyState instance = new IpReassemblyState();

    /**
        Called by IpReassemblyFromNetMessenger. This method in turn calls
        the messenger's reassemble() method which does all the work
    */
    public void applyFromNet(Message message, Session session,
			     Messenger messenger) {
    //+ IpReassemblyState::applyFromNet(Message, Session, Messenger)
      Message newMessage;
      Conduit nextConduit = message.getTransporter().getNext();
      IpReassemblySession sess = (IpReassemblySession) session;
      IpReassemblyFromNetMessenger msgr =
	(IpReassemblyFromNetMessenger) messenger;
      newMessage = msgr.reassemble(message, session);
      if (newMessage != null) {
	message = newMessage;
	MessageTransporter trans = 
	  MessageTransporter.newMessageTransporter(nextConduit, 
						   message, 
						   session);
	SimpleScheduler.instance.makeRunnable(trans);
      }
      message.setMessenger(IpHeaderFromNetMessenger.instance);
    //- IpReassemblyState::applyFromNet(Message, Session, Messenger)
    }

    /**
        Called by ReassemblyToNetMessenger. Does not call back on the
        messenger that called it. Instead it just sets the message's
        messenger pointer to point to the next messenger. (Yes, we are
        members of the Humphrey Appleby Fan Club)
        
    */
    public void applyToNet(Message message, Session session, Messenger messenger) {
    //+ IpReassemblyState::applyToNet(Message, Session, Messenger)
      message.setMessenger(IpRoutingToNetMessenger.instance);
    //- IpReassemblyState::applyToNet(Message, Session, Messenger)
    }

}; //IpReassemblyState

