/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingMessenger (implementation)
    Package: ip
    File:    IpRoutingMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;


/**
    IpRoutingMessenger
    
    This is the superclass of IpRoutingToNetMessenger and
    IpRoutingFromNetMessenger capturing their common methods. Provides the
    methods:
    1. setNextHopIpAddr()
    Retreives the next hop IP address based on the packet's destination
    address, and puts it into the message's OutOfBand data.
    2. destinationLocal()
    Tells whether the destination IP address is one of our local addresses.
    3. decrementTTL()
    Decrements the TTL field in the IP header, and returns the new value.
    
*/
public abstract class IpRoutingMessenger
  extends IpMessenger {

    public void setNextHopIpAddr(Message message, int ipAddr) {
    //+ IpRoutingMessenger::setNextHopIpAddr(Message, int)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();
      oob.setNextHopIpAddr(ipAddr);
    //- IpRoutingMessenger::setNextHopIpAddr(Message, int)
    }

    public void setInterfaceId(Message message, String id) {
    //+ IpRoutingMessenger::setInterfaceId(Message, String)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();
      oob.setInterfaceId(id);
    //- IpRoutingMessenger::setInterfaceId(Message, String)
    }

    public boolean destinationLocal(Message message, Session session) {
    //+ IpRoutingMessenger::destinationLocal(Message, Session)
      IpRoutingSession sess = (IpRoutingSession) session;
      int dest = getDestIpAddr(message);
      if (sess.interfaceTable.get(dest) == null) {
	return false;
      }
      return true;
    //- IpRoutingMessenger::destinationLocal(Message, Session)
    }

    public int decrementTTL(Message message) {
    //+ IpRoutingMessenger::decrementTTL(Message)
      BinaryBuffer buffer = message.getBuffer();
      byte newTTL = (byte) (buffer.getByte(8) - 1);
      // XXX Is there a need for additional timer-based decrement?
      buffer.setByte(8, newTTL);
      
      return (int) newTTL & 0xff;
    //- IpRoutingMessenger::decrementTTL(Message)
    }

}; //IpRoutingMessenger
