/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingOffState (implementation)
    Package: ip
    File:    IpRoutingOffState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpRoutingOffState
    
    The RoutingSession is in this state when routing is disabled.
*/
public class IpRoutingOffState
  extends IpRoutingState {

  public static final IpRoutingOffState instance = new IpRoutingOffState();


    /**
        Calls the Messenger's destinationLocal() method which tells if the
        destination IP address belongs to this machine. If the destination
        is local, then the message is forwarded to the ReassemblySession.
        Packets to nonlocal destinations are destroyed.
        
    */
    public void applyFromNet(Message message, Session session, Messenger messenger) {
    //+ IpRoutingOffState::applyFromNet(Message, Session, Messenger)
      IpRoutingFromNetMessenger msgr = (IpRoutingFromNetMessenger) messenger;
      if (!(msgr.destinationLocal(message, session))) {
	message.stopTransportation();
	int destIpAddr = msgr.getDestIpAddr(message);
	System.err.println("Packet forwarding disabled.");
	System.err.println("Discarding datagram addressed to " +
			   (((int) destIpAddr >> 24) & 0xff) + "." +
			   (((int) destIpAddr >> 16) & 0xff) + "." +
			   (((int) destIpAddr >>  8) & 0xff) + "." +
			   (((int) destIpAddr >>  0) & 0xff));
      }
      message.setMessenger(IpReassemblyFromNetMessenger.instance);
    //- IpRoutingOffState::applyFromNet(Message, Session, Messenger)
    }

}; //IpRoutingOffState

