/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  PseudoEthernetAdaptor (implementation)
    Package: ethernet
    File:    PseudoEthernetAdaptor.java

    Modifications by Esko Heimonen & Mikael Suokas, October 1997.
    Further modifications by Mikael Suokas, February 1998.
    Modifications to reflect new message format by Mikael Suokas, April 1998

    o tcp/ip stacks communicate through fifo:s within the same machine.
      No physical ethernet needed.
    o PseudoEthernetAdaptor has a main function, for testing purposes.
    o to run this, you need to run the companion fifoman.c program

*/
package fi.hut.tcm.jacob.tcpip.test;
// import fi.hut.tcm.jacob.conduits.ethernet.*;
import fi.hut.tcm.jacob.conduits.Adaptor;
import fi.hut.tcm.jacob.conduits.MediaAdaptor;
import fi.hut.tcm.jacob.conduits.SimpleScheduler;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.arp.*;

import java.io.*;

/**
    PseudoEthernetAdaptor
    
*/
public class PseudoEthernetAdaptor
  extends Adaptor
  implements Runnable {
    public static final byte[] ETHERTYPE_IP = new byte[] { (byte)0x08, (byte)0x00 };
    public static final byte[] ETHERTYPE_ARP = new byte[] { (byte)0x08, (byte)0x06 };
    public static final byte[] ETHERTYPE_RARP = new byte[] { (byte)0x80, (byte)0x35 };
    public static final byte[] ETHERTYPE_IPV6 = new byte[] { (byte)0x86, (byte)0xdd };
    public static final int ETHER_MTU = 1500;
    public static final int ETHER_ADDR_LEN = 6;
    private Thread adaptorThread;
    private String device;

    private String inName;    // name of input FIFO (without pathname)
    private String outName = "controlfifo";
    private byte[] addr;      // the ethernet address of this machine
    private FileInputStream inFD;
    private BufferedOutputStream outFD;

    int sendCnt = 0;
    int receiveCnt = 0;


    public PseudoEthernetAdaptor(byte[] myaddr) {
    //+ PseudoEthernetAdaptor::EthernetAdaptor(String, int)
      // our new stuff:
      addr = myaddr;
      inName =
 	Integer.toHexString(((int) myaddr[0])&0xff) + ":" +
        Integer.toHexString(((int) myaddr[1])&0xff) + ":" +
	Integer.toHexString(((int) myaddr[2])&0xff) + ":" + 
	Integer.toHexString(((int) myaddr[3])&0xff) + ":" +
	Integer.toHexString(((int) myaddr[4])&0xff) + ":0" + 
	Integer.toHexString(((int) myaddr[5])&0xff);
      System.out.print("Opening input file (" + inName + ")...");
      try {
	inFD = new FileInputStream(inName);
      } catch (FileNotFoundException e) {
	System.err.println("The file/fifo '" + inName +
			   "' was not found. Aborting.");
	System.exit(1);
      }
      System.out.print("Opening output file (" + outName + ")...");
      try {
	outFD = new BufferedOutputStream( new FileOutputStream(outName), 2048 );
	outFD.write(89);      // MAGIC_START - used to detect corruption
	outFD.write(2);       // 2 = opcode for "I am listening"
	for (int i = 0; i < 6; i++) {
	  outFD.write( (int) myaddr[i] & 0xff );
	}
	outFD.write(213);    // MAGIC_END - used to detect corruption
	outFD.flush();
      } catch (IOException e) {
	System.err.println("Error opening file/fifo '" + outName +
			   "' : " + e);
	System.exit(2);
      }
      System.out.println("done.");

      adaptorThread = new Thread(this);
      adaptorThread.start();

    //- PseudoEthernetAdaptor::EthernetAdaptor(String, int)
    }

    /**
        Stops the internal thread and releases the "physical" ethernet
        connection.
    */
    public void finalize() {
    //+ PseudoEthernetAdaptor::finalize( )
      // Kill the internal thread
      try {
        adaptorThread.stop();
        adaptorThread.join(100); // Wait until it is dead
      } catch (InterruptedException ex) {}
      adaptorThread = null;
      // now, tell the fifoman program that we are leaving
      try {
	outFD.write(89);                 
	outFD.write(3);                  // "opcode" for unregister function
	for (int i = 0; i < 6; i++) {
	  outFD.write(addr[i]);
	}
	outFD.write(213);
	outFD.flush();
	outFD.close();
	inFD.close();
      } catch (IOException e) {
	System.out.println("Problem closing down: " + e);
      }
    //- PseudoEthernetAdaptor::finalize( )
    }

    private int send(byte[] packet) {
    //+ PseudoEthernetAdaptor::send(byte[])
      // outFD is a BufferedOutputStream - hopefully it guarantees an atomic
      // write when flushed - otherwise serious corruption can occur when several
      // writers access the FIFO at the same time.
      try {
	outFD.write(89);                 // MAGIC_START - used to detect corruption
        outFD.write(1);                  // "opcode" for send function
        for (int i = 0; i < 6; i++) {    // destination address
          outFD.write( (int)packet[i] & 0xff );
        }
        outFD.write(packet.length >>> 8);
        outFD.write(packet.length & 0xff);
        outFD.write(packet,0,packet.length);
	outFD.write(213);                // MAGIC_END - used to detect corruption
        outFD.flush();
      } catch (IOException e) {
        System.err.println("Error writing: " + outName);
        System.exit(3);
      }

      System.out.print("EthernetAdaptor sent message #" + ++sendCnt );
      System.out.println(" to address " + (int) packet[5]);
      return packet.length;
    //- PseudoEthernetAdaptor::send(byte[])
    }

    private int receive(byte[] packet) {
    //+ PseudoEthernetAdaptor::receive(byte[])
      
      //  return receive(packet, 0, packet.length);
      // ** New, PseudoEthernetAdaptor code below **
      System.out.println("Listening...");
      try {
	int packetlen;
	if ( (packetlen = inFD.read()) == -1) {
	  System.err.println("*** EOF ***");
	  System.exit(0);
	}
	packetlen = 
	  ((packetlen << 8 ) &0x0000ff00) | 
	  (inFD.read() & 0x000000ff);
	inFD.read(packet,0,packetlen);
 	System.out.println("EthernetAdaptor received package #" + 
 			   ++receiveCnt + ":");
 	return packetlen;
       } catch (IOException e) {
 	System.err.println("Read error:" + e);
	System.exit(4);
      }

      return 0;
    //- PseudoEthernetAdaptor::receive(byte[])
    }

    public void adapt(Message message) {
    //+ PseudoEthernetAdaptor::adapt(Message)
      BinaryBuffer buf = message.getBuffer();
      byte[] packet = new byte[buf.getLength()];
      buf.exportData(packet, 0, buf.getLength());

      System.out.println("Visitor delivered message to PseudoEthernetAdaptor");

      send(packet);
    //- PseudoEthernetAdaptor::adapt(Message)
    }

    /**
        Called on adaptorThread. Tries to receive packets from the physical
        interface and insert them into the runqueue.
    */
    public void run() {
    //+ PseudoEthernetAdaptor::run( )
      System.out.println("Ethernet thread started");
      for (;;) {

	BinaryBuffer buf = 
	  BinaryBuffer.newBinaryBuffer(ETHER_MTU, 0, ETHER_MTU);
	byte[] packet = buf.getData();

	int len = receive(packet);

	long ethDst = buf.get6Bytes(0);
	long ethSrc = buf.get6Bytes(6);
	System.out.println("   dst: " + EthernetAddress.toString(ethDst) +
	                   " src: " + EthernetAddress.toString(ethSrc));

	buf.removeTrailer(ETHER_MTU - len);

	TcpipOutOfBand oob = new TcpipOutOfBand();

	Message message = 
	  new Message( EthernetHeaderFromNetMessenger.instance, oob, buf ); // XXX OK?
	if (sideA != null) {
	  MessageTransporter mt = 
	    MessageTransporter.newMessageTransporter(sideA, message, this);
	  SimpleScheduler.instance.makeRunnable(mt);
	}

      }
    //- PseudoEthernetAdaptor::run( )
    }

  public static void main(String[] args)           // for standalone testing
  {
    int i, j;
    PseudoEthernetAdaptor me;
    long my_address, dest_address;

    my_address = 0xaabbcc;
    my_address = my_address << 24;
    my_address += 0xddee00;
    dest_address = my_address;
    
    if (args.length < 1) {
      System.out.println("Usage: java PseudoEthernetAdaptor <own addr> " +
			 "[send addr]");
      System.out.println("The broadcast address is 255");
      System.exit(2);
    }

    Integer anInteger = Integer.valueOf(args[0]);
    i = anInteger.intValue();
    if ((i < 0) || (i > 9)) {
      System.out.println("Own address must be 0..9");
      System.exit(1);
    }
    my_address +=  i;

    me = new PseudoEthernetAdaptor( (new EthernetAddress(my_address)).toArray());
    System.out.println("Listening on address " + i);

    if (args.length > 1) {
      System.out.print("Waiting a while before sending packets...");
      try {
	Thread.sleep(5000);
      } catch (InterruptedException e) {
      }
      System.out.println("enough waiting.");
      i = Integer.valueOf(args[1]).intValue();
      if ( ((i < 0) || (i > 9)) && (i != 255) ) {
	System.out.println("Destination address must be 0..9 or 255");
	System.exit(1);
      }
      dest_address += i;

      BinaryBuffer buf = 
	BinaryBuffer.newBinaryBuffer(ETHER_MTU, 100, ETHER_MTU - 100);

      //byte[] temp = buf.data();
      //byte[] packet = new byte[temp.length ];
      //System.arraycopy(temp, 0, packet, 0, temp.length );

      buf.set6Bytes(6, my_address);
      buf.set6Bytes(0, dest_address);

      j = 1;
      for (;;) {
	System.out.print("Sending packet #" + j++ + " ");
	System.out.println(">>> dst: " + EthernetAddress.toString(buf.get6Bytes(0)) +
	                   " src: " + EthernetAddress.toString(buf.get6Bytes(6)) +
			   " >>>");
	try {
	Thread.sleep( (long) (java.lang.Math.random() * 3000) + 500 );
	} catch (InterruptedException e) {
	  System.exit(1);
	}

	// We would hope to avoid this kind of copying in the future:
	byte[] packet = new byte[buf.getLength()];
	buf.exportData(packet, 0, buf.getLength());
	me.send( packet );

	Thread.yield();
      }

    }

  }

}; // PseudoEthernetAdaptor
