/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  UdpChecksumFromNetMessenger (implementation)
    Package: udp
    File:    UdpChecksumFromNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.udp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    UdpChecksumFromNetMessenger
    
    
*/
public class UdpChecksumFromNetMessenger
  extends UdpMessenger {
    public static UdpChecksumFromNetMessenger instance = new UdpChecksumFromNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ UdpChecksumFromNetMessenger::apply(Message, State, Session)
      UdpChecksumState udpState = (UdpChecksumState) state; 
      udpState.applyFromNet(message, this);
    //- UdpChecksumFromNetMessenger::apply(Message, State, Session)
    }

    public boolean validChecksum(Message message) {
    //+ UdpChecksumFromNetMessenger::validChecksum(Message)

      BinaryBuffer buffer = message.getBuffer();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOutOfBand();

      int len = getLength(message);
      int srcIpAddr = oob.getSrcIpAddr();
      int destIpAddr = oob.getDestIpAddr();
      int csum = 0;

      // difference from UdpChecksumToNetMessenger: save the value first!
      int saved_checksum = getChecksum(message);
      if (saved_checksum == 0) {  // 0 = there is no checksum
	return true;
      }

      // calculations needs to have existing checksum field contains zeros:
      setChecksum(message, 0);

      // Calculate from a "pseudo header"
      csum += ((srcIpAddr >> 16) & 0xffff) + (srcIpAddr & 0xffff);
      csum += ((destIpAddr >> 16) & 0xffff) + (destIpAddr & 0xffff);
      csum += 17;  // zero field + UDP protocol id
      csum += len; // UDP length
      
      // Calculate from the datagram
      for (int i = 0; i < (len / 2) * 2; i += 2) {
	csum += buffer.getShort(i);
      }
      if ( (len % 2) == 1 ) {  // we had an odd size packet
	csum += buffer.getByte(len - 1);
      }

      // 32-bits to 16-bit folding and 1's complement
      while ((csum >> 16) > 0) {
        csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
      }
      csum = ~csum & 0xffff;

      if (saved_checksum == csum) {
	return true;
      } else {
	return false;
      }

    //- UdpChecksumFromNetMessenger::validChecksum(Message)
    }

}; //UdpChecksumFromNetMessenger

